#!/usr/bin/wish
# call the TCL/TK windowing shell
# Sinister, version 0.1

######################################
# legal stuff:
#
# Copyright 2004, 2005 Emanuele Ripamonti
#
# This file is part of Sinister.
#
# Sinister is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Sinister is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Sinister; if not, write to the
# Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
#
######################################

######################
# LIST OF PROCEDURES #
######################

proc write_message {x} {
    .message_fr.message_text insert end $x
    .message_fr.message_text see end
}

########################################

proc xaxis_clear {} {
    global xaxis_x1 xaxis_x2 xaxis_y1 xaxis_y2
    global xaxis_type

    set xaxis_x1 -1
    set xaxis_y1 -1
    set xaxis_x2 -1
    set xaxis_y2 -1
    set xaxis_type 0
    .main_fr.command_fr.x_fr.coord_fr.first_entry delete 0 end;
    .main_fr.command_fr.x_fr.coord_fr.first_entry insert 0 "---";
    .main_fr.command_fr.x_fr.coord_fr.second_entry delete 0 end;
    .main_fr.command_fr.x_fr.coord_fr.second_entry insert 0 "---";
    write_message "X-axis cleared\n"
}

########################################

proc yaxis_clear {} {
    global yaxis_x1 yaxis_x2 yaxis_y1 yaxis_y2
    global yaxis_type

    set yaxis_x1 -1
    set yaxis_y1 -1
    set yaxis_x2 -1
    set yaxis_y2 -1
    set yaxis_type 0
    .main_fr.command_fr.y_fr.coord_fr.first_entry delete 0 end;
    .main_fr.command_fr.y_fr.coord_fr.first_entry insert 0 "---";
    .main_fr.command_fr.y_fr.coord_fr.second_entry delete 0 end;
    .main_fr.command_fr.y_fr.coord_fr.second_entry insert 0 "---";
    write_message "Y-axis cleared\n"
}

########################################

proc browse {} {
    set types {
	{{GIF Files}        {.gif}        }
	{{All Files}        *             }
    }
    set id [tk_getOpenFile -filetypes $types -title "GIF file selection"];
    .file_fr.filename_entry delete 0 end;
    .file_fr.filename_entry insert 0 $id;
}

########################################

proc help_message {} {
    toplevel .help_window
    frame .help_window.top -bg LightBlue2

    text .help_window.top.text \
	-yscrollcommand ".help_window.top.yscroll set" \
	-height 25 -width 80
    scrollbar .help_window.top.yscroll -command ".help_window.top.text yview"
    button .help_window.done -text "Done" -command "destroy .help_window"

    set filehang [open "helptext.txt" "r"]
    set data [read $filehang]
    close $filehang

    .help_window.top.text insert 1.0 $data
    .help_window.top.text see 1.0


    pack .help_window.top.text .help_window.top.yscroll -side left
    pack configure .help_window.top.yscroll -expand yes -fill y
    pack .help_window.top .help_window.done

}

########################################

proc zoom_in {} {
    global zoom_level

    set zoom_level [expr $zoom_level+1]

    write_message "Zoom level is $zoom_level\n"
    load_image $zoom_level
}

########################################

proc zoom_out {} {
    global zoom_level

    set zoom_level [expr $zoom_level-1]

    write_message "Zoom level is $zoom_level\n"

    load_image $zoom_level
}

########################################

proc load_image {zoom} {
    global inputfile

    foreach id [.main_fr.canvas_fr.canvas find all] {
	.main_fr.canvas_fr.canvas delete $id;
    }
    write_message "Loading file $inputfile\n";
    image create photo original_image -file $inputfile;
    image create photo .main_fr.canvas_fr.bitmap
    if {($zoom>=0)} {
	.main_fr.canvas_fr.bitmap copy original_image -zoom [expr $zoom+1];
    } else {
	.main_fr.canvas_fr.bitmap copy original_image -subsample [expr 1-$zoom];
    }

    .main_fr.canvas_fr.canvas create image 0 0 \
	    -image .main_fr.canvas_fr.bitmap -anchor nw;
}

########################################

proc xaxis_extr {} {
    global xaxis_x1 xaxis_y1 xaxis_x2 xaxis_y2
    global x1 y1 x2 y2

    write_message "Setting the X-Axis\nClick 1st X-axis point\n"
    set x1 -1 ; set y1 -1 ; set x2 -1 ; set y2 -1
    
    bind .main_fr.canvas_fr.canvas <Button-1> { \
	    if {($x1==-1)&&($y1==-1)} { \
	    set x1 %x; set y1 %y; \
	    write_message "X-axis 1st point: %x %y\nClick 2nd X-axis point\n"} \
	    else { \
	    set x2 %x; set y2 %y; \
	    write_message "X-axis 2nd point: %x %y\n"; \
	    set xaxis_x1 $x1; set xaxis_x2 $x2; \
	    set xaxis_y1 $y1; set xaxis_y2 $y2; \
	    bind .main_fr.canvas_fr.canvas <Button-1> {}; \
	    bind .main_fr.canvas_fr.canvas <Button-2> {}; \
	    bind .main_fr.canvas_fr.canvas <Button-3> {}; } }
    
    bind .main_fr.canvas_fr.canvas <Button-3> { \
	    write_message "1st point erased\nClick 1st X-axis point\n"; \
	    set x1 -1; set y1 -1; set x2 -1; set y2 -1}
    
    bind .main_fr.canvas_fr.canvas <Button-2> { \
	    write_message "Set-X-Axis operation cancelled\n"; \
	    bind .main_fr.canvas_fr.canvas <Button-1> {}; \
	    bind .main_fr.canvas_fr.canvas <Button-2> {}; \
	    bind .main_fr.canvas_fr.canvas <Button-3> {}; }

}

########################################

proc yaxis_extr {} {
    global yaxis_x1 yaxis_y1 yaxis_x2 yaxis_y2
    global x1 y1 x2 y2

    write_message "Setting the Y-Axis\nClick 1st Y-axis point\n"
    set x1 -1 ; set y1 -1 ; set x2 -1 ; set y2 -1
    
    bind .main_fr.canvas_fr.canvas <Button-1> { \
	    if {($x1==-1)&&($y1==-1)} { \
	    set x1 %x; set y1 %y; \
	    write_message "Y-axis 1st point: $x1 $y1\nClick 2nd Y-axis point\n"} \
	    else { \
	    set x2 %x; set y2 %y; \
	    write_message "Y-axis 2nd point: $x2 $y2\n"; \
	    set yaxis_x1 $x1; set yaxis_x2 $x2; \
	    set yaxis_y1 $y1; set yaxis_y2 $y2; \
	    bind .main_fr.canvas_fr.canvas <Button-1> {}; \
	    bind .main_fr.canvas_fr.canvas <Button-2> {}; \
	    bind .main_fr.canvas_fr.canvas <Button-3> {}; } }
    
    bind .main_fr.canvas_fr.canvas <Button-3> { \
	    write_message "1st point erased\nClick 1st Y-axis point\n"; \
	    set x1 -1; set y1 -1; set x2 -1; set y2 -1}
    
    bind .main_fr.canvas_fr.canvas <Button-2> { \
	    write_message "Set-Y-Axis operation cancelled\n"; \
	    bind .main_fr.canvas_fr.canvas <Button-1> {}; \
	    bind .main_fr.canvas_fr.canvas <Button-2> {}; \
	    bind .main_fr.canvas_fr.canvas <Button-3> {}; }
}

########################################

proc get_points {} {
    global x_point y_point 
    global lastpoint
    global xcoord ycoord
    
    set i $lastpoint+1
    bind .main_fr.canvas_fr.canvas <Button-1> { \
	    set x1 %x; set y1 %y; \
	    set xcoord 0.; set ycoord 0.; \
	    write_message "X: %x\t Y: %y\n"; \
	    set xcoord [transform_x $x1 $y1]; \
	    set ycoord [transform_y $x1 $y1]; \
	    write_message "Xc: $xcoord\t Yc: $ycoord\n"; \
	    incr lastpoint; \
	    set x_point($lastpoint) $xcoord; \
	    set y_point($lastpoint) $ycoord; }
    
    bind .main_fr.canvas_fr.canvas <Button-3> { \
	    write_message "Last point erased\n"; \
	    unset x_point($lastpoint); \
	    unset y_point($lastpoint); \
	    incr lastpoint -1}
    
    bind .main_fr.canvas_fr.canvas <Button-2> { \
	    write_message "Point insertion ended\n"; \
	    bind .main_fr.canvas_fr.canvas <Button-1> {}; \
	    bind .main_fr.canvas_fr.canvas <Button-2> {}; \
	    bind .main_fr.canvas_fr.canvas <Button-3> {}; }
}

########################################

proc transform_x {x1 y1} {
    global xaxis_x1 xaxis_y1 xaxis_x2 xaxis_y2
    global x1coord x2coord xcoord
    global yaxis_x1 yaxis_y1 yaxis_x2 yaxis_y2
    global y1coord y2coord ycoord
    global xaxis_type

    if {($xaxis_x1<0) || ($x1coord==$x2coord) || ($xaxis_x1==$xaxis_x2)} {
	set x2 $x1
    } else {
	if {($yaxis_x1>=0)&&($yaxis_x1 != $yaxis_x2)} {
	    set mx [expr ($xaxis_y2-$xaxis_y1+0.0)/($xaxis_x2-$xaxis_x1+0.0)]
	    set my [expr ($yaxis_y2-$yaxis_y1+0.0)/($yaxis_x2-$yaxis_x1+0.0)]
	    set qx [expr $xaxis_y1-($mx*$xaxis_x1+0.0)]
	    set qy [expr $y1-($my*$x1+0.0)]
	    set x1 [expr ($qy-$qx+0.0)/($mx-$my+0.0)]
	}

	if {($xaxis_type==0)} {
	    set x2 [expr $x1coord+ \
			((($x1-$xaxis_x1+0.0)/($xaxis_x2-$xaxis_x1+0.0))* \
			     ($x2coord-$x1coord+0.0))]
	} else {
	    set x2 [expr $x1coord* \
			exp((($x1-$xaxis_x1+0.0)/($xaxis_x2-$xaxis_x1+0.0))* \
			     (log($x2coord)-log($x1coord)+0.0))]
	}
    }
    return $x2
}

########################################

proc transform_y {x1 y1} {
    global xaxis_x1 xaxis_y1 xaxis_x2 xaxis_y2
    global x1coord x2coord xcoord
    global yaxis_x1 yaxis_y1 yaxis_x2 yaxis_y2
    global y1coord y2coord ycoord
    global yaxis_type

    if {($yaxis_x1<0) || ($y1coord==$y2coord) || ($yaxis_y1==$yaxis_y2)} {
	set y2 $y1
    } else {
	if {($xaxis_x1>=0)&&($xaxis_y1!=$xaxis_y2)} {
	    set mx [expr ($xaxis_x2-$xaxis_x1+0.0)/($xaxis_y2-$xaxis_y1+0.0)]
	    set my [expr ($yaxis_x2-$yaxis_x1+0.0)/($yaxis_y2-$yaxis_y1+0.0)]
	    set qx [expr $x1-($mx*$y1+0.0)]
	    set qy [expr $yaxis_x1-($my*$yaxis_y1+0.0)]
	    set y1 [expr ($qy-$qx)/($mx-$my+0.0)]
	} 

	set y2 [expr $y1coord+ \
		((($y1-$yaxis_y1+0.0)/($yaxis_y2-$yaxis_y1+0.0))* \
		($y2coord-$y1coord+0.0))]

	if {($yaxis_type==0)} {
	    set y2 [expr $y1coord+ \
		   ((($y1-$yaxis_y1+0.0)/($yaxis_y2-$yaxis_y1+0.0))* \
		   ($y2coord-$y1coord+0.0))]
	} else {
	    set y2 [expr $y1coord* \
		   exp((($y1-$yaxis_y1+0.0)/($yaxis_y2-$yaxis_y1+0.0))* \
		   (log($y2coord)-log($y1coord)+0.0))]
	}

    }
    return $y2
}

########################################

proc clear_points {} {
    global x_point y_point 
    global lastpoint

    if {($lastpoint>0)} {
	unset x_point
	unset y_point
	set lastpoint 0
    }
    write_message "Points cleared\n"
}

########################################

proc save_points {} {
    global x_point y_point 
    global lastpoint

    if {$lastpoint<=0} {
	write_message "No points to save!\n"
    } \
	    else {
	set types {
	    {{All Files}        *             }
	}
	set id [tk_getSaveFile -filetypes $types \
		-title "Output file selection"];
	set outputfilename $id
	write_message "Saving to file $id\n"
	set fileid [open $outputfilename w]

	for {set i 1} {$i<=$lastpoint} {incr i} {
	    set x1 $x_point($i)
	    set y1 $y_point($i)
	    puts $fileid "$x1\t$y1"
	}
	close $fileid
    }
}

########################################

################
# MAIN PROGRAM #
################

wm title . "Sinister"
. configure -bg LightBlue2
#  Set the title of the application window and set a LightBlue2 background

########################################
# Frame definitions
# in the top we put a frame with the input file commands + Quit
frame .file_fr -bg LightBlue2

# in the center we have the canvas where to put the image
# and the buttons/frames with relevant commands
frame .main_fr -bg LightBlue2
frame .main_fr.canvas_fr -bg LightBlue2
frame .main_fr.command_fr -bg LightBlue2

# frame for messages in the bottom
frame .message_fr -bg LightBlue2

########################################
# definition of widgets inside .file_fr

# label and entry for manual editing of the filename
label .file_fr.filename_lbl -text Filename: -bg LightBlue2 -fg black
entry .file_fr.filename_entry -textvariable inputfile -width 40 \
	-bg white -fg black
.file_fr.filename_entry insert 0 "example.gif"

# button for browsing the file to load
button .file_fr.browse -text Browse.. -command {browse}

# button for actually loading the file
button .file_fr.load -text Load -command {load_image $zoom_level}

# button for quitting
button .file_fr.quit -text Quit -command {exit}

# button for help
button .file_fr.help -text Help -command {help_message}

####################
# definition of the canvas
canvas .main_fr.canvas_fr.canvas -bg LightBlue1 -cursor crosshair ;#-width 512 -height 512 
#image create photo .main_fr.canvas_fr.bitmap -file $inputfile
#.main_fr.canvas_fr.canvas create image 0 0 -image .main_fr.canvas_fr.bitmap -anchor nw


####################
# widgets inside the command_fr (.main_fr.command_fr)

# frames and buttons for the x-axis definition
frame .main_fr.command_fr.x_fr -bg LightBlue2
frame .main_fr.command_fr.x_fr.type_fr
frame .main_fr.command_fr.x_fr.button_fr
frame .main_fr.command_fr.x_fr.coord_fr

label .main_fr.command_fr.x_fr.title_lbl -text X-Axis

label .main_fr.command_fr.x_fr.type_fr.type_lbl -text Type:
radiobutton .main_fr.command_fr.x_fr.type_fr.lin \
	-text Lin -value 0 -variable xaxis_type
radiobutton .main_fr.command_fr.x_fr.type_fr.log \
	-text Log -value 1 -variable xaxis_type
set xaxis_type 0

button .main_fr.command_fr.x_fr.button_fr.extr \
	-text "Extremes" -command {xaxis_extr}
button .main_fr.command_fr.x_fr.button_fr.clear \
	-text "Clear" -command {xaxis_clear}

label .main_fr.command_fr.x_fr.coord_fr.first_lbl -text X1
entry  .main_fr.command_fr.x_fr.coord_fr.first_entry \
	-textvariable x1coord -width 8 -bg white -fg black
.main_fr.command_fr.x_fr.coord_fr.first_entry insert 0 "---"
label .main_fr.command_fr.x_fr.coord_fr.second_lbl -text X2
entry .main_fr.command_fr.x_fr.coord_fr.second_entry \
	-textvariable x2coord -width 8 -bg white -fg black
.main_fr.command_fr.x_fr.coord_fr.second_entry insert 0 "---"

# frames and buttons for the y-axis definition
frame .main_fr.command_fr.y_fr -bg LightBlue2
frame .main_fr.command_fr.y_fr.type_fr
frame .main_fr.command_fr.y_fr.button_fr
frame .main_fr.command_fr.y_fr.coord_fr

label .main_fr.command_fr.y_fr.title_lbl -text Y-Axis

label .main_fr.command_fr.y_fr.type_fr.type_lbl -text Type:
radiobutton .main_fr.command_fr.y_fr.type_fr.lin \
	-text Lin -value 0 -variable yaxis_type
radiobutton .main_fr.command_fr.y_fr.type_fr.log \
	-text Log -value 1 -variable yaxis_type
set yaxis_type 0

button .main_fr.command_fr.y_fr.button_fr.extr \
	-text "Extremes" -command {yaxis_extr}
button .main_fr.command_fr.y_fr.button_fr.clear \
	-text "Clear" -command {yaxis_clear}

label .main_fr.command_fr.y_fr.coord_fr.first_lbl -text Y1
entry  .main_fr.command_fr.y_fr.coord_fr.first_entry \
	-textvariable y1coord -width 8 -bg white -fg black
.main_fr.command_fr.y_fr.coord_fr.first_entry insert 0 "---"
label .main_fr.command_fr.y_fr.coord_fr.second_lbl -text Y2
entry .main_fr.command_fr.y_fr.coord_fr.second_entry \
	-textvariable y2coord -width 8 -bg white -fg black
.main_fr.command_fr.y_fr.coord_fr.second_entry insert 0 "---"

# zoom commands
frame .main_fr.command_fr.zoom_fr -bg LightBlue2
button .main_fr.command_fr.zoom_fr.zoom_in \
        -text "Zoom In" -command {zoom_in}
button .main_fr.command_fr.zoom_fr.zoom_out \
        -text "Zoom Out" -command {zoom_out}

# other commands
button .main_fr.command_fr.get_points \
	-text "Get Points" -command {get_points}
button .main_fr.command_fr.clear_points \
	-text "Clear Points" -command {clear_points}
button .main_fr.command_fr.save_points \
	-text "Save Points" -command {save_points}


####################
# widgets inside .message_fr
text .message_fr.message_text -bg white -fg black -height 6\
	-yscrollcommand ".message_fr.yscroll set"


scrollbar .message_fr.yscroll -command ".message_fr.message_text yview"


#bind .main_fr.canvas_fr.canvas <Button-1> {write_message "%x %y\n"}
########################################
# initialization commands
set zoom_level 0
load_image $zoom_level

########################################
# pack widgets
pack .file_fr .main_fr .message_fr -side top -pady 5 -expand yes -fill both
pack configure .file_fr -expand no
pack configure .message_fr -expand no

pack .file_fr.filename_lbl .file_fr.filename_entry \
	.file_fr.browse .file_fr.load .file_fr.quit .file_fr.help \
	-side left -padx 2


pack .main_fr.command_fr .main_fr.canvas_fr -side left \
    -padx 10 -expand yes -fill both
pack configure .main_fr.command_fr -expand no

pack .main_fr.canvas_fr.canvas -expand yes -fill both

pack .main_fr.command_fr.x_fr \
	.main_fr.command_fr.y_fr \
        .main_fr.command_fr.zoom_fr \
	.main_fr.command_fr.get_points \
	.main_fr.command_fr.clear_points \
	.main_fr.command_fr.save_points \
	-side top -pady 5

pack .main_fr.command_fr.x_fr.title_lbl \
	.main_fr.command_fr.x_fr.type_fr \
	.main_fr.command_fr.x_fr.button_fr \
	.main_fr.command_fr.x_fr.coord_fr \
	-side top -pady 2

pack .main_fr.command_fr.x_fr.type_fr.type_lbl \
	.main_fr.command_fr.x_fr.type_fr.lin \
	.main_fr.command_fr.x_fr.type_fr.log \
	-side left -padx 2

pack .main_fr.command_fr.x_fr.button_fr.extr \
	.main_fr.command_fr.x_fr.button_fr.clear \
	-side left -padx 5

pack .main_fr.command_fr.x_fr.coord_fr.first_lbl \
	.main_fr.command_fr.x_fr.coord_fr.first_entry \
	.main_fr.command_fr.x_fr.coord_fr.second_lbl \
	.main_fr.command_fr.x_fr.coord_fr.second_entry \
	-side left -padx 2

pack .main_fr.command_fr.y_fr.title_lbl \
	.main_fr.command_fr.y_fr.type_fr \
	.main_fr.command_fr.y_fr.button_fr \
	.main_fr.command_fr.y_fr.coord_fr \
	-side top -pady 2

pack .main_fr.command_fr.y_fr.type_fr.type_lbl \
	.main_fr.command_fr.y_fr.type_fr.lin \
	.main_fr.command_fr.y_fr.type_fr.log \
	-side left -padx 2

pack .main_fr.command_fr.y_fr.button_fr.extr \
	.main_fr.command_fr.y_fr.button_fr.clear \
	-side left -padx 5

pack .main_fr.command_fr.zoom_fr.zoom_in \
        .main_fr.command_fr.zoom_fr.zoom_out \
        -side left -padx 5

pack .main_fr.command_fr.y_fr.coord_fr.first_lbl \
	.main_fr.command_fr.y_fr.coord_fr.first_entry \
	.main_fr.command_fr.y_fr.coord_fr.second_lbl \
	.main_fr.command_fr.y_fr.coord_fr.second_entry \
	-side left -padx 2

pack .message_fr.message_text .message_fr.yscroll -side left
pack configure .message_fr.message_text -expand yes -fill x
pack configure .message_fr.yscroll -expand yes -fill y

set lastpoint 0









